import { html } from "uhtml";
import { signal } from "uhtml/preactive";
import {
  checkServiceWorkers,
  deregisterServiceWorkers,
} from "./settings/Settings";
import { Workbox } from "workbox-window";

const open = signal(false);
const registration = signal<ServiceWorkerRegistration | undefined>(undefined);

export const UpdateModal = () => {
  return html` <dialog ?open=${open.value}>
    <article class="container-fluid">
      <h3>An update is available!</h3>
      <p>
        A new version of the app is available! Click the button below to update
        and reload. If you're stuck in an endless update loop, click Clear Cache
        below to fix it.
      </p>
      <p>
        <button aria-label="Update and reload" @click=${update}>Update</button>
      </p>
      <p>
        <button
          class="secondary"
          aria-label="Clear Cache"
          @click=${deregisterServiceWorkers}
        >
          Clear Cache
        </button>
      </p>
    </article>
  </dialog>`;
};

const update = async (event: Event) => {
  event.preventDefault();
  if (registration.value && registration.value.waiting) {
    console.log("Service Worker is updating.");
    registration.value.waiting.postMessage({ type: "SKIP_WAITING" });
  } else {
    console.error("Service Worker failed to update.");
  }
};

export const checkForUpdates = async () => {
  try {
    if ("serviceWorker" in navigator) {
      const wb = new Workbox("/sw.js");

      wb.addEventListener("waiting", () => {
        console.log("Service Worker is ready to handle offline usage.");
        open.value = true;
      });

      wb.addEventListener("controlling", () => {
        window.location.reload();
      });

      registration.value = await wb.register();

      console.log("Service Worker has been registered.");

      await checkServiceWorkers();
    }
  } catch (err) {
    console.error("Failed to register Service Worker: ", err);
  }
};
