import { html } from "uhtml";
import {
  exportKeys,
  importAndSaveKeys,
  retrieveOrGenerateKeyPair,
} from "../../utils/crypto.ts";
import { signal } from "uhtml/preactive";
import { LOCAL_STORAGE_KEYS } from "../../utils/store.ts";
import { openSettings } from "./Settings.ts";

const ecdhPublicKey = signal<string | null>(null);
const ecdhPrivateKey = signal<string | null>(null);

export const KeyManager = () => html`
   <h2>Key Manager</h2>
   <p>
     You can export your keys to transfer to a different device, or import keys exported from a different device.
   </p>
   <p>
     Public Key:
   <pre>${ecdhPublicKey}</pre>
   </p>
   <p>
     Private Key:
   <pre>${ecdhPrivateKey}</pre>
   </p>
   <p>
     <button @click=${exportKeys}>Export Keys</button>
     <button @click=${importKeys}>Import Keys</button>
     <input type="file" accept=".json" style="display: none" id="keypairImportEl" @change=${handleKeypairImport} />
     <button class="secondary" @click=${openSettings}>Back</button>
   </p>`;

function importKeys(event: Event): void {
  const target = event.target as HTMLElement;
  const el = target.nextElementSibling as HTMLInputElement;
  el.click();
}

function handleKeypairImport(event: Event): void {
  const fileInput = event.target as HTMLInputElement;
  const file = fileInput.files?.[0];
  if (!file) return;

  const reader = new FileReader();
  reader.onload = async (event) => {
    const keys = JSON.parse(event.target?.result as string);
    await importAndSaveKeys(keys);

    alert("Successfully imported keypair");
  };
  reader.readAsText(file);
}

export async function initKeyManager() {
  await retrieveOrGenerateKeyPair();
  ecdhPublicKey.value = localStorage.getItem(
    LOCAL_STORAGE_KEYS.ECDH_PUBLIC_KEY
  );
  ecdhPrivateKey.value = localStorage.getItem(
    LOCAL_STORAGE_KEYS.ECDH_PRIVATE_KEY
  );
  return true;
}
