import { html } from "uhtml";
import { closeModal, openModal } from "../Modal.ts";
import { NotificationManager } from "./NotificationManager.ts";
import { KeyManager, initKeyManager } from "./KeyManager.ts";
import { hasServiceWorkers } from "../../utils/store.ts";

export const Settings = () => html`<h2>Settings</h2>
  <section>
    <button @click=${openKeyManager}>Manage Keys</button>
    <button @click=${openNotificationManager}>Notifications</button>
    <button
      @click=${deregisterServiceWorkers}
      ?disabled=${!hasServiceWorkers.value}
    >
      Clear Cache
    </button>
    <button class="secondary" @click=${closeModal}>Close</button>
  </section>`;

export function openSettings(event: Event) {
  event.preventDefault;
  openModal(Settings);
}

async function openKeyManager(event: Event) {
  event.preventDefault();
  await initKeyManager();
  openModal(KeyManager);
}

async function openNotificationManager(event: Event) {
  event.preventDefault();
  openModal(NotificationManager);
}

export async function checkServiceWorkers() {
  if ("serviceWorker" in navigator) {
    const registrations = await navigator.serviceWorker.getRegistrations();
    hasServiceWorkers.value = registrations.length > 0;
    console.log("Service Workers registered:", hasServiceWorkers.value);
  }
}

export async function deregisterServiceWorkers(event: Event) {
  event.preventDefault();
  if ("serviceWorker" in navigator) {
    const registrations = await navigator.serviceWorker.getRegistrations();
    await Promise.all(
      registrations.map((registration) => registration.unregister())
    );
  }
  await checkServiceWorkers();
  alert(
    "Service Workers deregistered! If you think you're not seeing the latest version of the app, try refreshing the page."
  );
}
