import "./style.scss";
import { page } from "./utils/store.ts";
import { Header } from "./components/Header.ts";
import { html } from "uhtml";
import { effect } from "uhtml/preactive";
import { reactive } from "uhtml/reactive";
import { Footer } from "./components/Footer.ts";
import { HowItWorks } from "./components/HowItWorks.ts";
import { FAQ } from "./components/FAQ.ts";
import { routeToPage, hashChange } from "./router.ts";
import { UpdateModal, checkForUpdates } from "./components/Update.ts";
import { subscribeUserToPush } from "./components/settings/NotificationManager.ts";
import { AppModal } from "./components/Modal.ts";

try {
  const render = reactive(effect);
  const template = () => html` ${AppModal()} ${UpdateModal()}
    <main class="container">
      <section>${routeToPage(page.value)}</section>
      <div>${HowItWorks}</div>
      <div>${FAQ}</div>
    </main>
    ${Footer}`;

  // Listen for changes in the hash and update the state accordingly
  window.addEventListener("hashchange", hashChange);

  // Initialize the app
  hashChange();
  checkForUpdates();
  const headerElement = document.getElementById("header");
  render(headerElement, Header);

  const appElement = document.getElementById("page");
  render(appElement, template);

  if (Notification.permission === "granted") subscribeUserToPush();
} catch (error) {
  console.error(error);
}
